module RelCalcTest

/*
 * authors: 
 * 	Miguel Ferreira		<pg10961@alunos.uminho.pt>
 * 	Samuel Silva   		<pg11034@alunos.uminho.pt>
 * description: 
 * 		
 */

/*******************************************************************/

open util/relation as rel
open RelCalc

abstract sig Test {}
sig Test1, Test2 extends Test {}

/*******************************************************************/
-- wait a minute
assert predicatesOK{
  all R: Test -> Test | {
    RelCalc/dom[R] = rel/dom[R]
    RelCalc/rng[R] = rel/ran[R]
    RelCalc/Function[R] <=> rel/function[R, rel/dom[R]]
    RelCalc/Entire[R] <=> rel/total[R, rel/dom[R]]
    RelCalc/Simple[R] <=> rel/functional[R, rel/dom[R]]
    RelCalc/Surjective[R] <=> rel/surjective[R, rel/ran[R]]
    RelCalc/Injective[R] <=> rel/injective[R, rel/ran[R]]
    RelCalc/Reflexive[R] <=> rel/reflexive[R, rel/dom[R] + rel/ran[R]]
    RelCalc/Bijection[R] <=> rel/bijection[R, rel/dom[R], rel/ran[R]]
    RelCalc/Symmetric[R] <=> rel/symmetric[R]
    RelCalc/Antisymmetric[R] <=> rel/antisymmetric[R]
    RelCalc/Transitive[R] <=> rel/transitive[R]
    RelCalc/Acyclic[R] <=> rel/acyclic[R, rel/dom[R]]
    RelCalc/Preorder[R] <=> rel/preorder[R, rel/dom[R] + rel/ran[R]]
    RelCalc/Equivalence[R] <=> rel/equivalence[R, rel/dom[R]]
    RelCalc/Partialorder[R] <=> rel/partialOrder[R, rel/dom[R] + rel/ran[R]]
   }
}

check predicatesOK for 10
/*******************************************************************/
/**Relcalc predicates*/
assert BasicStructure{
  all R, S, T, U: Test -> Test | {
--    T.(S.R) = (T.S).R
    R = correflexive[univ].R 
    R = R.(correflexive[univ])

    R in R
    R in S and S in T => R in T
    R in S and S in R => R = S
    
    R in T and R in U => R.S in U.T
  }
}

check BasicStructure for 10

/*******************************************************************/

assert RelationalTaxonomy{
  all R: Test -> Test | {
--    Reflexive[R] <=> R.univ <:iden in R
    Correflexive[R] <=> R in iden :> univ.R
    Transitive[R] <=> R.R in R
    Symmetric[R] <=> R in ~R
    Antisymmetric[R] <=> R & ~R in correflexive[univ]
    Connected[R] <=> R + ~R in Univ[univ,univ]
    Entire[R] <=> R.univ <: iden in ker[R]
    Simple[R] <=> img[R] in (iden :> univ.R)
    Surjective[R] <=> Entire[~R]
    Injective[R] <=> Simple[~R]
  }
}

check RelationalTaxonomy for 10

/*******************************************************************/

assert PFTransformation{
  all f, g: Test -> Test | {
--    Function[f] and Function[g] 
  }
}

check PFTransformation for 10

/*******************************************************************/

assert GaloisConnection{
  all X,Y: Test -> Test, f, g: Test -> Test | Function[f] && Function[g] => {
    ~X in Y <=> X in ~Y
--    X.f in Y <=> X in Y.(~g)
--    (~f).X in Y <=> X in g.Y
--    X in (f.g)[X]
--    (g.f)[Y] in Y
  }
}

check GaloisConnection for 10

/*******************************************************************/

assert OtherGaloisConnections{
  all X,R,S: Test -> Test | {
    X in R & S <=> (X in R) and (X in S)
    (R+S) in X <=> (R in X) and (S in X)
  }
}

check OtherGaloisConnections for 10

/*******************************************************************/

assert AlmostGaloisConnections{
  all R,S,T: Test -> Test | {
--    R.S in T <=> R.(correflexive[RelCalc/dom[S]]) in T.~S
--    ~S.R in T <=> (correflexive[RelCalc/dom[S]]).R in S.T
--    correflexive[RelCalc/dom[R]] in T <=> R in T.R
--    correflexive[RelCalc/rng[R]] in T <=> R in R.T
  }
}

check AlmostGaloisConnections for 10

/*******************************************************************/

assert Converses{
  all R,S: Test -> Test | {
    R in ~R <=> R = ~R
    R in R.~R.R
    ~(~R) = R
    ~(S.R) = (~R.~S)
    ~(R&S) = (~R) & (~S)
  }
}

check Converses for 10

/*******************************************************************/

assert Dedekind{
  all R,S,T: Test -> Test | {
--    (S.R) & T in (S & (T.~R)).R
--    (S.R) & T in S.(R & (~S.T))
--    (S.R) & T in (S & T.~R) . (R & (~S.T)) 
	
--    (S.~R) & correflexive[univ] in (R & S) . ~(R & S)
--    R & T in (correflexive[univ] & (T.~R)) . R
  }
}

check Dedekind for 10

/*******************************************************************/

assert Correflexives{
  all X,Y,R,S: Test -> Test | {
    Correflexive[X] => (X = ~X) and (X = X . X) and (X = X & correflexive[univ])
    Correflexive[X] and Correflexive[Y] => X.Y = X & Y
    Correflexive[X] => (R.X in S <=> R.X in S.X)
    Correflexive[X] => (X.R in S <=> X.R in X.S)
  }
}

check Correflexives for 10

/*******************************************************************/

assert RelationalDivisions{
  all R, S, T: Test -> Test | {
  }
}

check RelationalDivisions for 10

/*******************************************************************/

assert Meets{
  all R,S,T: Test -> Test, F: Test -> Test | {
    T.ker[R] in T or S.ker[R] in S => (S&T).R = (S.R) & (T.R)
    img[R] . T in T or S.(img[R]) in S => R.(S&T) = (R.S) & (R.T) 
    Function[F] => F.(S&T) = (F.S)&(F.T)
	
    (R & S) & T = R & (S & T)
    R & S = S & R
    R & R = R
    R in S <=> R = S & S
    R & S in R and R & S in S
    (R & S).T in R.T & S.T
    T.(R&S) in T.R & T.S
  }
}

check Meets for 10

/*******************************************************************/

assert KernelAndImage{
  all R,S: Test -> Test | {
    ker[R] = R.~R
    R in S => ker[R] in ker[S]
    ~(ker[R]) = ker[R]
    R in ker[R].R
    ker[S.R] = S.(ker[R]).(~S)
    (S.~R) & id[univ] = ker[R & correflexive[univ]]
	
    img[R] = ker[~R]
  }
}

check KernelAndImage for 10

/*******************************************************************/

assert DomainAndRange{
  all R,S: Test -> Test | {
    correflexive[RelCalc/dom[R]] = ker[R] & correflexive[univ]
    R = correflexive[RelCalc/dom[R]].R
--    correflexive[RelCalc/dom[S.R]] = 
--    RelCalc/correflexive[RelCalc/dom[S.(RelCalc/correflexive[RelCalc/dom[R]])]]
    correflexive[RelCalc/dom[R&S]] = (S.~R) & RelCalc/correflexive[univ]
	
    RelCalc/dom[R] = RelCalc/rng[~R]
  }
}

check DomainAndRange for 10

/*******************************************************************/

assert Splits {
}

check Splits for 10

/*******************************************************************/

assert Eithers {
}

check Eithers for 10

/*******************************************************************/

assert RelationalProjection {
}

check RelationalProjection for 10

/*******************************************************************/

assert Equality{
  all R,S, X: Test -> Test | {
    R = S <=> R in S and S in R
	
--    R = S <=> X in R and X in S
--    R = S <=> R in X and S in X
  }
}

check Equality for 10

/*******************************************************************/

/***********************************************/

/*	let r = univ -> univ | {
--		correflexive[r] <=> and per[r]
		reflexive[r] <=> (not bottom[r]) and cotransitive[r]
		reflexive[r] and correflexive[r] <=> correflexive[r]
		id[r] => bijection[r]
		id[Id[univ]] => bijection[Id[univ]]
	} */

/*******************************************************************/
